@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
@@ -49,6 +50,7 @@
 -- > import Data.Time.Clock
 -- > import Data.Time.Calendar
 -- > import qualified Control.Exception as E
+-- > import Network.HTTP.Types.Status (statusCode)
 -- >
 -- > past :: UTCTime
 -- > past = UTCTime (ModifiedJulianDay 56200) (secondsToDiffTime 0)
@@ -73,9 +75,9 @@
 -- > main = withSocketsDo $ do
 -- >      request' <- parseUrl "http://example.com/secret-page"
 -- >      let request = request' { cookieJar = Just $ createCookieJar [cookie] }
--- >      E.catch (withManager $ httpLbs request)
+-- >      (fmap Just (withManager $ httpLbs request)) `E.catch`
 -- >              (\(StatusCodeException s _ _) ->
--- >                if statusCode==403 then putStrLn "login failed" else return ())
+-- >                if statusCode s==403 then (putStrLn "login failed" >> return Nothing) else return Nothing)
 --
 -- Any network code on Windows requires some initialization, and the network
 -- library provides withSocketsDo to perform it. Therefore, proper usage of
@@ -89,8 +91,8 @@
 -- >
 -- > main = withSocketsDo
 -- >      $ simpleHttp "http://www.haskell.org/" >>= L.putStr
--- >
--- > Cookies are implemented according to RFC 6265.
+--
+-- Cookies are implemented according to RFC 6265.
 --
 -- Note that by default, the functions in this package will throw exceptions
 -- for non-2xx status codes. If you would like to avoid this, you should use
@@ -122,6 +124,9 @@
 -- >     let settings = mkManagerSettings (TLSSettingsSimple True False False) Nothing
 -- >     res <- withManagerSettings settings $ httpLbs request
 -- >     print res
+--
+-- For more information, please be sure to read the documentation in the
+-- "Network.HTTP.Client" module.
 
 module Network.HTTP.Conduit
     ( -- * Perform a request
@@ -150,6 +155,7 @@ module Network.HTTP.Conduit
     , responseTimeout
     , cookieJar
     , getConnectionWrapper
+    , HCC.setQueryString
       -- *** Request body
     , requestBodySource
     , requestBodySourceChunked
@@ -199,7 +205,7 @@ module Network.HTTP.Conduit
 
 import qualified Data.ByteString              as S
 import qualified Data.ByteString.Lazy         as L
-import           Data.Conduit                 (ResumableSource, ($$+-), await, ($$++), ($$+), Source)
+import           Data.Conduit                 (ResumableSource, ($$+-), await, ($$++), ($$+), Source, addCleanup)
 import qualified Data.Conduit.Internal        as CI
 import qualified Data.Conduit.List            as CL
 import           Data.IORef                   (readIORef, writeIORef, newIORef)
@@ -243,6 +249,11 @@ import           Network.HTTP.Client.Internal (Cookie (..), CookieJar (..),
 -- usage, you'll need to use @conduit@ packages's
 -- 'C.Source' returned by 'http'.
 --
+-- This function will 'throwIO' an 'HttpException' for any
+-- response with a non-2xx status code (besides 3xx redirects up
+-- to a limit of 10 redirects). This behavior can be modified by
+-- changing the 'checkStatus' field of your request.
+--
 -- Note: Unlike previous versions, this function will perform redirects, as
 -- specified by the 'redirectCount' setting.
 httpLbs :: MonadIO m => Request -> Manager -> m (Response L.ByteString)
@@ -303,9 +314,15 @@ http :: MonadResource m
      -> m (Response (ResumableSource m S.ByteString))
 http req man = do
     (key, res) <- allocate (Client.responseOpen req man) Client.responseClose
+#if MIN_VERSION_conduit(1, 2, 0)
     let rsrc = CI.ResumableSource
-            (HCC.bodyReaderSource $ responseBody res)
+            (flip CI.unConduitM CI.Done $ addCleanup (const $ release key) $ HCC.bodyReaderSource $ responseBody res)
             (release key)
+#else
+    let rsrc = CI.ResumableSource
+            (addCleanup (const $ release key) $ HCC.bodyReaderSource $ responseBody res)
+            (release key)
+#endif
     return res { responseBody = rsrc }
 
 requestBodySource :: Int64 -> Source (ResourceT IO) S.ByteString -> RequestBody
